#ifndef DRAW_DIALOG_H_INC
#define DRAW_DIALOG_H_INC

#include <map>
#include <utility>

inline uint32_t MakeLong(uint16_t aLow, uint16_t aHigh)
{
   return aLow | (aHigh << 16);
}

class ModalDialogClass
{
protected:
   ModalDialogClass()
   {
   }

   virtual ~ModalDialogClass()
   {
   }

   static UINT DoDialog(void *aObj, HWND aParent, UINT aResourceID)
   {
      return DialogBoxParam(AppClass::GetApp()->GetInstanceHandle(), MAKEINTRESOURCE(aResourceID), aParent, (DLGPROC )DialogProc, (LPARAM )aObj);
   }

   virtual void OnInitDialog() = 0;
   virtual bool OnClose(UINT aMSG) = 0;
   virtual bool OnCommand(UINT aId, UINT aMsg, HWND aCtrlHwnd)
   {
      return TRUE;
   }
   virtual void OnHScroll(uint16_t aCode, uint16_t aPos, HWND aCtrlHWnd)
   {
   }

   static UINT DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
   {
      ModalDialogClass *Dialog = 0;
      DialogMapClass::iterator idx = DialogMap.find(hwndDlg);
      if(idx == DialogMap.end())
      {
         if(msg != WM_INITDIALOG)
            return TRUE;
      }
      else
      {
         Dialog = idx->second;
      }

      switch (msg)
      {
      case WM_COMMAND:
         switch (LOWORD(wParam))
         {
         case IDOK:
            if(Dialog->OnClose(IDOK))
            {
               EndDialog(hwndDlg, 1);
               DialogMap.erase(idx);
            }
            return TRUE;
         case IDCANCEL:
            if(Dialog->OnClose(IDCANCEL))
            {
               EndDialog(hwndDlg, 0);
               DialogMap.erase(idx);
            }
            return TRUE;
         default:
            Dialog->OnCommand(LOWORD(wParam), HIWORD(wParam), (HWND )lParam);
         }
         return TRUE;
      case WM_INITDIALOG:
      {
         ModalDialogClass *Obj = (ModalDialogClass *)lParam;
         DialogMap.insert(std::make_pair(hwndDlg, Obj));
         Obj->WindowHandle = hwndDlg;
         Obj->OnInitDialog();
         return TRUE;
      }
      case WM_CLOSE:
         if(Dialog->OnClose(WM_CLOSE))
         {
            EndDialog(hwndDlg, 0);
            DialogMap.erase(idx);
         }
         return TRUE;
      case WM_HSCROLL:
         Dialog->OnHScroll(LOWORD(wParam), HIWORD(wParam), (HWND )lParam);
         return TRUE;
      }
      return FALSE;
   }

private:
   typedef std::map<HWND, ModalDialogClass *>DialogMapClass;
   static DialogMapClass DialogMap;

protected:
   HWND WindowHandle;

   // Some handy utility functions
   HWND GetDlgItem(int id)
   {  return ::GetDlgItem(WindowHandle, id); }
   LRESULT SendDlgItemMessage(int id, UINT Msg, WPARAM wParam, LPARAM lParam)
   {  return ::SendDlgItemMessage(WindowHandle, id, Msg, wParam, lParam); }
   bool SetDlgItemText(int id, const char *text)
   {  return ::SetDlgItemText(WindowHandle, id, text); }
   bool SetDlgItemText(int id, std::string str)
   {  return ::SetDlgItemText(WindowHandle, id, str.c_str()); }
   bool GetDlgItemText(int id, char *text, size_t Size)
   {  return ::GetDlgItemText(WindowHandle, id, text, Size); }
   bool SetDlgItemInt(int id, uint32_t Value)
   {  return ::SetDlgItemInt(WindowHandle, id, Value, FALSE); }

   bool GetDlgItemText(int id, std::string &str)
   {
      HWND Item = GetDlgItem(id);
      if(!Item)
         return FALSE;
      int Length = GetWindowTextLength(Item) + 1;
      char *Temp = new char [Length];

      if(!::GetWindowText(Item, Temp, Length))
      {
         delete [] Temp;
         return FALSE;
      }
      str = Temp;
      delete [] Temp;
      return TRUE;
   }
   bool GetDlgItemInt(int id, uint32_t &Value)
   {
      BOOL Success;
      Value = ::GetDlgItemInt(WindowHandle, id, &Success, FALSE);
      return Success;
   }

   bool GetDlgItemInt(int id, int32_t &Value)
   {
      BOOL Success;
      Value = (int32_t )::GetDlgItemInt(WindowHandle, id, &Success, FALSE);
      return Success;

   }
};

#endif
