#ifndef IMAGE_H_INC
#define IMAGE_H_INC
#include <windows.h>
#include <stdio.h>
#include <string>
#include "drawstd.h"

extern int ReadJpegFile(const char *aFilename, uint32_t **&aData, int32_t &aWidth, int32_t &aHeight, bool &IsMono, std::string &ErrorString);
extern void WriteJpegFile(const char *aFilename, uint32_t **&aData, int32_t &aWidth, int32_t &aHeight, int aQuality);

class ImageClass
{
public:
   ImageClass()
   {
      Data = 0;
   }
   ~ImageClass()
   {
     delete [] Data;
   }

   uint8_t *GetData() { return Data; };
   bool Read(const char *aFileName);

   bool WriteJpg(const char *aFileName, int aQuality);
   bool WriteBmp(const char *aFileName);
   bool CreateFromClipboard(void *aPtr);
   bool CreateFromScreenDump();

   HANDLE CreateClipboardData();
   int32_t GetWidth() const {return Width;}
   int32_t GetHeight() const {return Height;}

   void Resize(int32_t aNewWidth, int32_t aNewHeight);
   void Mirror();
   void Rotate();
   void Crop(int32_t aLeft, int32_t aTop, int32_t aRight, int32_t aBottom);
   void Move(int32_t aXOffset, int32_t aYOffset, COLORREF aFillColor);
   bool operator == (const ImageClass &rhs);
   bool operator != (const ImageClass &rhs);

   void InitializBitmapInfoHeader(BITMAPINFOHEADER &aBitmapFileHeader);
   std::string ErrorStr;
   void WriteBitmapFileHeader(FILE *aFile, BITMAPFILEHEADER &aBitmapFileHeader);
   void ReadBitmapFileHeader(class ReaderBaseClass &aReader, BITMAPFILEHEADER &aBitmapFileHeader);

protected:
   uint32_t BytesPerLine;

   uint8_t GetAt(int32_t aY, int32_t aX, int32_t aIdx) const
   {
      return Data[aY*BytesPerLine + aX*3 + aIdx];
   }

   bool ReadBmp(class ReaderBaseClass &aReader);
   int32_t Width;
   int32_t Height;
   uint8_t *Data;
   std::string FileName;
   uint8_t *ReadBmpPalette(class ReaderBaseClass &Reader, int32_t &Width, int32_t Height, uint32_t Colors, uint32_t BitPixel, std::string &ErrStr);
};

#endif
