#ifndef IMAGEWIN_H_INC
#define IMAGEWIN_H_INC

#include "drawstd.h"
#include "image.h"
#include "scroll.h"

class DrawAppClass;

class ImageWindowClass : public ScrollWindowClass
{
public:
   ImageWindowClass(ImageClass *Image, COLORREF BkColor) : ScrollWindowClass(), NormalZoom(1.0)
   {
      Parent = 0;
      DispOffsetX = 0;
      DispOffsetY = 0;
      CurrentImage = Image;
      Zoom = NormalZoom;
      BackColor = BkColor;
      IsRect = false;
      memset(&FrameRect, 0, sizeof(FrameRect));
      memset(&ImageRect, 0, sizeof(ImageRect));
      IsFraming = false;
      IsMoving = 0;
   }
   ~ImageWindowClass()
   {
      delete CurrentImage;
   }
   virtual void MoveWindow(RECT &aRect, bool ReDraw = true);
   void SetImage(ImageClass *Image);
   void UpdateImage();
   void SetZoom();
   void ZoomToFit();
   void Resize();
   void Rotate();
   void Mirror();
   void Crop();

   virtual void OnLButtonDown(uint32_t aFlags, int16_t aXPos, int16_t aYPos);
   virtual void OnLButtonUp(uint32_t aFlags, int16_t aXPos, int16_t aYPos);
   virtual void OnRButtonDown(uint32_t aFlags, int16_t aXPos, int16_t aYPos);
   virtual void OnMouseMove(uint32_t aFlags, int16_t aXPos, int16_t aYPos);

   void SetBackColor(COLORREF Color)
   {
      BackColor = Color;
      InvalidateRect(WindowHandle, 0, TRUE);
   }

   ImageClass *GetImage()
   {
      return CurrentImage;
   }
   BOOL Create(DrawAppClass *_Parent, uint32_t Width, uint32_t Height);
   LRESULT OnEraseBackground(HDC dc)
   {
      return TRUE;
   }

protected:
   virtual LRESULT OnPaint(void);
   void ShowImage(HDC dc);
   bool OnCommand(int id, HWND hwndCtl, UINT codeNotify);
private:
   int32_t GetDispWidth(void);
   int32_t GetDispHeight(void);
   int32_t DispOffsetX;
   int32_t DispOffsetY;
   int32_t MovingX;
   int32_t MovingY;
   DrawAppClass *Parent;
   const double NormalZoom;
   double Zoom;
   ImageClass *CurrentImage;
   COLORREF BackColor;
   bool IsRect;
   RECT FrameRect; // The on-screen selected rect
   RECT ImageRect; // The on-image selected rect
   RECT ImageRectFromFrameRect(RECT &aRect);
   RECT FrameRectFromImageRect(RECT &aRect);
   void RemoveRect();
   bool IsFraming;
   bool IsMoving;
   POINT FrameStart;
   void CalcFrameRect(int32_t aX, int32_t aY);
};

#endif
