#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include <commctrl.h>
#include <sstream>
#include <string>
#include "drawres.h"
#include "main.h"
#include "imagewin.h"
#include "jpgoptdialog.h"
#include <fstream>

DrawAppClass::DrawAppClass(HINSTANCE hInstance, const char *aCommandLine) :
   WindowBaseClass(), AppClass(), CmdLine(aCommandLine), Registry("Software\\MyCompany\\JDraw")
{
   InstanceHandle = hInstance;
   WindowHandle = 0;
   Me = this;
   ImageWindow = 0;
   StatusBarHeight = 0;
   FileNameIndex = 0;

   BackColor = Registry.GetInt("BackColor", RGB(255, 255, 255));
   WindowRect.left = Registry.GetInt("Left", 10);
   WindowRect.top = Registry.GetInt("Top", 10);
   WindowRect.right = Registry.GetInt("Right", 400);
   WindowRect.bottom = Registry.GetInt("Bottom", 200);
}

DrawAppClass::~DrawAppClass()
{
   delete ImageWindow;
   Registry.WriteInt("BackColor", BackColor);

   Registry.WriteInt("Left", WindowRect.left);
   Registry.WriteInt("Top", WindowRect.top);
   Registry.WriteInt("Right", WindowRect.right);
   Registry.WriteInt("Bottom", WindowRect.bottom);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpCmdLine, INT)
{
   InitCommonControls();

   DrawAppClass DrawApp(hInstance, lpCmdLine);
   DrawApp.Create();
   return DrawApp.Run();
}

bool DrawAppClass::OnCommand(int aId, HWND, UINT)
{
   switch(aId)
   {
   case IDM_EXIT:
      PostMessage(WindowHandle, WM_CLOSE, 0, 0);
      break;
   case IDM_OPEN:
      Open();
      break;
   case IDM_SAVE_BMP:
      SaveBMP();
      break;
   case IDM_SAVE_JPG:
      SaveJPG();
      break;
   case IDM_PASTE_CLIP:
      PasteClip();
      break;
   case IDM_COPY_CLIP:
      CopyClip();
      break;
   case IDM_SET_BK_COLOR:
      SetBkColor();
      break;
   case IDM_NEXT_IMAGE:
      NextImage();
      break;
   case IDM_PREV_IMAGE:
      PrevImage();
      break;
   case IDM_SCREEN_DUMP:
      ScreenDump();
      break;
   case IDM_ABOUT:
      MessageBox(WindowHandle, "Bitmap and JPeg Image Viewer Version 2.0\r\nNo copyright", "JDraw", MB_OK);
      break;
   default:
      return FALSE;
   }
   return TRUE;
}

LRESULT DrawAppClass::OnCreate(CREATESTRUCT *aCreateStruct)
{
   HICON Icon = LoadIcon(InstanceHandle, MAKEINTRESOURCE(IDAPPLICON));
   SendMessage(WindowHandle, WM_SETICON, ICON_BIG, (LPARAM )Icon);
   SendMessage(WindowHandle, WM_SETICON, ICON_SMALL, (LPARAM )Icon);
   CreateStatusBar();
   if(!CmdLine.empty())
   {
      DoOpen(CmdLine.c_str());
   }
   return 0;
}

bool DrawAppClass::Create()
{
   WNDCLASS wc;

   memset(&wc, 0, sizeof(WNDCLASS));
   wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
   wc.lpfnWndProc = (WNDPROC )&DrawAppClass::WndProcEntry;
   wc.hInstance = InstanceHandle;
   wc.hbrBackground = (HBRUSH )(COLOR_APPWORKSPACE + 1);
   wc.lpszClassName = "DrawWndClass";
   wc.lpszMenuName = MAKEINTRESOURCE(IDMAINMENU);
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
   wc.hIcon = LoadIcon(NULL, MAKEINTRESOURCE(IDAPPLICON));
   if(!RegisterClass(&wc))
      return FALSE;
   WindowHandle = CreateWindow("DrawWndClass",
                               "Draw",
                               WS_MINIMIZEBOX | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_MAXIMIZEBOX | WS_CAPTION | WS_BORDER | WS_SYSMENU | WS_THICKFRAME,
                               WindowRect.left, WindowRect.top, WindowRect.right - WindowRect.left, WindowRect.bottom - WindowRect.top,
                               // CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
                               NULL,
                               NULL,
                               InstanceHandle,
                               (LPVOID )this);
   return TRUE;
}

int DrawAppClass::Run()
{
   MSG msg;
   HACCEL hAccelTable;

   hAccelTable = LoadAccelerators(InstanceHandle, MAKEINTRESOURCE(IDACCEL));

   ShowWindow(WindowHandle, SW_SHOW);
   while(GetMessage(&msg, NULL, 0, 0))
   {
      if(!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
      {
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      }
   }
   return msg.wParam;
}

void DrawAppClass::Open(void)
{
   static char Filter[] = "jpeg (*.jpg,*.jpeg)\0*.jpg;*.jpeg\0Bitmaps (*.bmp)\0*.bmp\0All (*.*)\0*.*\0";

   char *TName = new char [16*1024];
   TName[0] = 0;
   OPENFILENAME OpenFilename;

   memset(&OpenFilename, 0, sizeof(OpenFilename));
   OpenFilename.lStructSize = sizeof(OpenFilename);
   OpenFilename.hwndOwner = WindowHandle;
   OpenFilename.hInstance = InstanceHandle;
   OpenFilename.lpstrFilter = Filter;
   OpenFilename.lpstrFile = TName;
   OpenFilename.nMaxFile = 16*1024;
   OpenFilename.Flags = OFN_FILEMUSTEXIST | OFN_EXPLORER | OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY;

   if(GetOpenFileName(&OpenFilename))
   {
      FileNameList.clear();
      FileNameIndex = 0;
      std::string Path = TName;
      char *s = TName + strlen(TName) + 1;
      if(*s == 0)
      {  // Only one file
         FileNameList.push_back(Path);
      }
      else
      {
         do
         {
            std::string T = Path;
            T += "\\";
            T += s;
            FileNameList.push_back(T);
            s += strlen(s) + 1;
         }
         while(*s != 0);
      }
      CurrentFileName = *FileNameList.begin();
      DoOpen(CurrentFileName.c_str());
   }
   delete [] TName;
}

void DrawAppClass::NextImage()
{
   if(!FileNameList.size())
      return;
   if(FileNameIndex < FileNameList.size() - 1)
   {
      FileNameIndex++;
      CurrentFileName = FileNameList[FileNameIndex];
      DoOpen(CurrentFileName.c_str());
   }
}

void DrawAppClass::PrevImage()
{
   if(!FileNameList.size() || !FileNameIndex)
      return;
   FileNameIndex--;
   CurrentFileName = FileNameList[FileNameIndex];
   DoOpen(CurrentFileName.c_str());
}

void DrawAppClass::DoOpen(const char *aFileName)
{
   ImageClass *Image = new ImageClass;
   WaitCursorClass WaitCursor;

   if(Image->Read(aFileName))
   {
      if(!ImageWindow)
      {
         ImageWindow = new ImageWindowClass(Image, BackColor);
         RECT ClientRect = GetClientRect();
         ImageWindow->Create(this, ClientRect.right, ClientRect.bottom - StatusBarHeight);
      }
      else
         ImageWindow->SetImage(Image);
      UpdateStatus();
   }
   else
   {
      MessageBox(WindowHandle, Image->ErrorStr.c_str(), "JDraw", MB_OK);
      delete Image;
   }
}

void DrawAppClass::UpdateStatus()
{
   if(!ImageWindow || !ImageWindow->GetImage())
      return;
   std::stringstream ss;
   ss <<  ImageWindow->GetImage()->GetWidth() << "*" << ImageWindow->GetImage()->GetHeight();
   UpdateStatusBar(ss.str().c_str(), 1, 0);
   std::string st = "JDraw - [";
   st += CurrentFileName;
   st += "]";
   SetWindowText(WindowHandle, st.c_str());
}

void DrawAppClass::SaveBMP()
{
   if(!ImageWindow || !ImageWindow->GetImage())
      return;

   static char Filter[] = "bmp (*.bmp)\0*.bmp\0All (*.*)\0*.*\0";
   char Drive[256];
   char Path[256];
   char FN[256];
   _splitpath(CurrentFileName.c_str(), Drive, Path, FN, 0);
   strcat(Drive, Path);
   strcat(Drive, FN);
   strcat(Drive, ".bmp");
   OPENFILENAME OpenFilename;

   memset(&OpenFilename, 0, sizeof(OpenFilename));
   OpenFilename.lStructSize = sizeof(OpenFilename);
   OpenFilename.hwndOwner = WindowHandle;
   OpenFilename.hInstance = InstanceHandle;
   OpenFilename.lpstrFilter = Filter;
   OpenFilename.lpstrFile = Drive;
   OpenFilename.nMaxFile = sizeof(Drive);
   OpenFilename.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;
   OpenFilename.lpstrDefExt = "bmp";

   if(GetSaveFileName(&OpenFilename))
   {
      WaitCursorClass WaitCursor;
      ImageWindow->GetImage()->WriteBmp(Drive);
      ImageWindow->GetImage()->Read(Drive);  // Read the file we just wrote
      InvalidateRect(WindowHandle, 0, TRUE);
      if(FileNameList.size() <= FileNameIndex)
         FileNameList.push_back(Drive);
      else
         FileNameList[FileNameIndex] = Drive;
      CurrentFileName = Drive;
      UpdateStatus();
   }
}

void DrawAppClass::SaveJPG()
{
   if(!ImageWindow || !ImageWindow->GetImage())
      return;

   JpgOptionsDialogClass JpgOptionsDialog(WindowHandle);
   if(JpgOptionsDialog.Run())
   {
      static char Filter[] = "jpeg (*.jpg,*.jpeg)\0*.jpg;*.jpeg\0All (*.*)\0*.*\0";
      char Drive[256];
      char Path[256];
      char FN[256];
      _splitpath(CurrentFileName.c_str(), Drive, Path, FN, 0);
      strcat(Drive, Path);
      strcat(Drive, FN);
      strcat(Drive, ".jpg");
      OPENFILENAME OpenFilename;

      memset(&OpenFilename, 0, sizeof(OpenFilename));
      OpenFilename.lStructSize = sizeof(OpenFilename);
      OpenFilename.hwndOwner = WindowHandle;
      OpenFilename.hInstance = InstanceHandle;
      OpenFilename.lpstrFilter = Filter;
      OpenFilename.lpstrFile = Drive;
      OpenFilename.nMaxFile = sizeof(Drive);
      OpenFilename.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;
      OpenFilename.lpstrDefExt = "jpg";

      if(GetSaveFileName(&OpenFilename))
      {
         WaitCursorClass WaitCursor;
         ImageWindow->GetImage()->WriteJpg(Drive, JpgOptionsDialog.GetQuality());
         ImageWindow->GetImage()->Read(Drive);  // Read the file we just wrote
         InvalidateRect(WindowHandle, 0, TRUE);

         if(FileNameList.size() <= FileNameIndex)
            FileNameList.push_back(Drive);
         else
            FileNameList[FileNameIndex] = Drive;
         CurrentFileName = Drive;
         UpdateStatus();
      }
   }
}

void DrawAppClass::PasteClip()
{
   HANDLE handle;

   if(!OpenClipboard(NULL))
   {
     MessageBox(WindowHandle, "Failed to open clipboard!", "JDraw", MB_OK);
     return;
   }
   if((handle = GetClipboardData(CF_DIB)) == 0)
   {
     MessageBox(WindowHandle, "Failed to get clipboard data", "JDraw", MB_OK);
     CloseClipboard();
     return;
   }

   void *p;
   if((p = GlobalLock(handle)) == NULL)
   {
     MessageBox(WindowHandle, "Failed to lock clipboard data", "JDraw", MB_OK);
     CloseClipboard();
     return;
   }

   ImageClass *CurrentImage = new ImageClass;
   if(CurrentImage->CreateFromClipboard(p))
   {
      if(!ImageWindow)
      {
         ImageWindow = new ImageWindowClass(CurrentImage, BackColor);
         RECT ClientRect = GetClientRect();
         ImageWindow->Create(this, ClientRect.right, ClientRect.bottom - StatusBarHeight);
      }
      else
         ImageWindow->SetImage(CurrentImage);

      CurrentFileName = "Unknown.ext";
      UpdateStatus();
   }
   else
   {
      delete CurrentImage;
   }
}

void DrawAppClass::ScreenDump()
{
   ImageClass *CurrentImage = new ImageClass;
   if(CurrentImage->CreateFromScreenDump())
   {
      if(!ImageWindow)
      {
         ImageWindow = new ImageWindowClass(CurrentImage, BackColor);
         RECT ClientRect = GetClientRect();
         ImageWindow->Create(this, ClientRect.right, ClientRect.bottom - StatusBarHeight);
      }
      else
         ImageWindow->SetImage(CurrentImage);

      CurrentFileName = "Unknown.ext";
      UpdateStatus();
   }
   else
   {
      delete CurrentImage;
   }
}

void DrawAppClass::CopyClip()
{
   if(!ImageWindow || !ImageWindow->GetImage())
      return;

   if(!OpenClipboard(NULL))
   {
     MessageBox(WindowHandle, "Failed to open clipboard!", "JDraw", MB_OK);
     return;
   }
   if(!EmptyClipboard())
   {
      MessageBox(WindowHandle, "Failed to empty clipboard", "JDraw", MB_OK);
      CloseClipboard();
      return;
   }
   HANDLE Handle = ImageWindow->GetImage()->CreateClipboardData();

   if(!SetClipboardData(CF_DIB, Handle))
      MessageBox(WindowHandle, "Failed to set clipboard data", "JDraw", MB_OK);
   CloseClipboard();
}

void DrawAppClass::SetBkColor()
{
   CHOOSECOLOR cc;

   memset(&cc, 0, sizeof(cc));
   cc.lStructSize = sizeof(cc);
   cc.hwndOwner = WindowHandle;
   cc.rgbResult = BackColor;
   COLORREF CustColor[16];
   for(int i = 0; i < 16; i++)
     CustColor[i] = RGB(0xFF, 0xFF, 0xFF);
   cc.lpCustColors = CustColor;
   cc.Flags = CC_ANYCOLOR | CC_RGBINIT | CC_SOLIDCOLOR;
   if(ChooseColor(&cc))
   {
      BackColor = cc.rgbResult;
      if(ImageWindow)
         ImageWindow->SetBackColor(BackColor);
   }
}

void DrawAppClass::OnMsgMenuSelect(uint16_t aCmd, uint16_t aFlags, HMENU aMenu)
{
   char Buffer[256];
   UINT   StringID;

   Buffer[0] = 0;
   if(aFlags == 0xffff && aMenu == NULL)
      StringID = 0;
   else if(aFlags & MFT_SEPARATOR)
      StringID = 0;
   else if(aFlags & MF_POPUP)
   {
      if(aFlags & MF_SYSMENU)
         StringID = IDS_SYSMENU;
      else
         StringID = 0;
   }
   else
      StringID = aCmd;

   if(StringID)
       LoadString(InstanceHandle, StringID, Buffer, sizeof(Buffer));
   UpdateStatusBar(Buffer, 0, SBT_NOBORDERS);
}

void DrawAppClass::SetStatusBarWidths()
{
   int32_t SBarWidths[4];
   RECT ClientRect = GetClientRect();
   HDC dc = GetDC(StatusbarWindowHandle);
   uint32_t Style = GetWindowLong(WindowHandle, GWL_STYLE);

   SIZE TextWidth;
   GetTextExtentPoint32(dc, "9999*9999", 9, &TextWidth);

   SBarWidths[3] = ClientRect.right;
   if(!(Style & WS_MAXIMIZE))
   {  // Must take the gripper into account
      if(!StatusBarHeight  && IsWindow(StatusbarWindowHandle))
      {
         RECT SBarRect;
         GetWindowRect(StatusbarWindowHandle, &SBarRect);
         StatusBarHeight = uint16_t(SBarRect.bottom - SBarRect.top);
      }
      SBarWidths[3] -= StatusBarHeight;
   }
   SBarWidths[2] = SBarWidths[3] - TextWidth.cx;
   SBarWidths[1] = SBarWidths[2] - TextWidth.cx;
   SBarWidths[0] = SBarWidths[1] - TextWidth.cx;
   SendMessage(StatusbarWindowHandle, SB_SETPARTS, 4, (LPARAM )SBarWidths);
}

void DrawAppClass::OnSize(uint32_t aType, uint16_t aX, uint16_t aY)
{
   SendMessage(StatusbarWindowHandle, WM_SIZE, aType, MakeLong(aX, aY));
   SetStatusBarWidths();
   RECT CR = GetClientRect();
   CR.bottom -= StatusBarHeight;
   if(ImageWindow)
      ImageWindow->MoveWindow(CR);
}

void DrawAppClass::OnClose()
{
   GetWindowRect(WindowHandle, &WindowRect);
}

void DrawAppClass::CreateStatusBar()
{
   StatusbarWindowHandle = CreateStatusWindow(WS_CHILD | WS_VISIBLE | WS_BORDER | SBARS_SIZEGRIP,
                                              "Ready",
                                              WindowHandle,
                                              IDR_STATUSBAR);
   SetStatusBarWidths();
   UpdateStatusBar("Ready", 0, SBT_NOBORDERS);
   UpdateStatusBar("   ", 1, 0);
   UpdateStatusBar("   ", 2, 0);
   UpdateStatusBar("   ", 3, 0);
}

void DrawAppClass::SetZoomSize(uint32_t aWidth, uint32_t aHeight)
{
   std::stringstream ss;
   ss << aWidth << " * " << aHeight;
   UpdateStatusBar(ss.str().c_str(), 2, 0);
}

void DrawAppClass::SetImageSize(uint32_t aWidth, uint32_t aHeight)
{
   std::stringstream ss;
   ss << aWidth << "*" << aHeight;
   UpdateStatusBar(ss.str().c_str(), 1, 0);
}

void DrawAppClass::SetFrameSize(uint32_t aWidth, uint32_t aHeight)
{
   std::stringstream ss;
   if(aWidth != 0 || aHeight != 0)
      ss << aWidth << "*" << aHeight;
   UpdateStatusBar(ss.str().c_str(), 3, 0);
}

AppClass *AppClass::Me;
ModalDialogClass::DialogMapClass ModalDialogClass::DialogMap;
WindowBaseClass::WindowMapClass WindowBaseClass::WindowMap;
HCURSOR WaitCursorClass::Cursor;
//--- EOF
