#ifndef DRAW_MAIN_H_INC
#define DRAW_MAIN_H_INC
#include <string>
#include <sstream>
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <stdio.h>
#include <map>
#include <vector>
#include "basewin.h"

class AppClass
{
public:
   HINSTANCE GetInstanceHandle() {return InstanceHandle; }
   static AppClass *GetApp() {return Me;}

protected:
   AppClass()
   {
      Me = this;
   }
   static AppClass *Me;
   HINSTANCE InstanceHandle;
};

class WaitCursorClass
{
public:
   WaitCursorClass()
   {
      if(!Cursor)
         Cursor = LoadCursor(0, IDC_WAIT);
      PrevCursor = SetCursor(Cursor);
   }
   ~WaitCursorClass()
   {
      SetCursor(PrevCursor);
   }
private:
   static HCURSOR Cursor;
   HCURSOR PrevCursor;
};

class RegistryClass
{
public:
   RegistryClass(const char *aKey)
   {
      if(RegOpenKey(HKEY_CURRENT_USER, aKey, &KeyHandle) != ERROR_SUCCESS)
      {
         if(RegCreateKeyEx(HKEY_CURRENT_USER, aKey, 0, 0, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0, &KeyHandle, 0) != ERROR_SUCCESS)
            KeyHandle = 0;
      }
   }
   ~RegistryClass()
   {
      if(KeyHandle)
         RegCloseKey(KeyHandle);
   }
   int GetInt(const char *aKey, int aDefaultValue = 0)
   {
      if(!KeyHandle)
        return aDefaultValue;

      DWORD Type = REG_DWORD, Size = sizeof(DWORD);
      int Value;
      if(RegQueryValueEx(KeyHandle, aKey, 0, &Type, (BYTE *)&Value, &Size) != ERROR_SUCCESS)
         return aDefaultValue;

      return Value;
   }

   void WriteInt(const char *aKey, int aValue)
   {
      if(!KeyHandle)
         return;
      RegSetValueEx(KeyHandle, aKey, 0, REG_DWORD, (BYTE *)&aValue, sizeof(aValue));
   }

private:
   HKEY KeyHandle;
};

class DrawAppClass : public WindowBaseClass, AppClass
{
public:
   DrawAppClass(HINSTANCE hInstance, const char *CommandLine) ;
   ~DrawAppClass();
   bool Create();
   int Run();
   virtual void OnSize(uint32_t aType, uint16_t aX, uint16_t aY);
   virtual void OnClose();
   void SetFrameSize(uint32_t aWidth, uint32_t aHeight);
   void SetZoomSize(uint32_t aWidth, uint32_t aHeight);
   void SetImageSize(uint32_t aWidth, uint32_t aHeight);

   static HWND GetWindowHandle()
   {
      return ((DrawAppClass *)Me)->WindowHandle;
   }
private:
   void Open(void);
   void DoOpen(const char *aFileName);
   void NextImage();
   void PrevImage();
   void UpdateStatus();
   virtual bool OnCommand(int id, HWND hwndCtl, UINT codeNotify);
   virtual LRESULT OnCreate(CREATESTRUCT *CreateStruct);

   void PasteClip();
   void CopyClip();
   void SaveBMP();
   void SaveJPG();
   void ScreenDump();
   void SetBkColor();

   void UpdateStatusBar(const char *aStr, uint16_t aPartNumber, uint16_t aDisplayFlags)
   {
      SendMessage(StatusbarWindowHandle, SB_SETTEXT, aPartNumber | aDisplayFlags, (LPARAM )aStr);
   }

   void CreateStatusBar();

   void SetStatusBarWidths();

   virtual void OnMsgMenuSelect(uint16_t aCmd, uint16_t aFlags, HMENU aMenu);

   std::string CmdLine;
   std::string CurrentFileName;
   HWND StatusbarWindowHandle;
   class ImageWindowClass *ImageWindow;
   uint16_t StatusBarHeight;

   COLORREF BackColor;
   RegistryClass Registry;
   RECT WindowRect;
   std::vector <std::string> FileNameList;
   std::vector <std::string>::size_type FileNameIndex;
};

#endif
