#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include <commctrl.h>
#include <string>

#include "scroll.h"

void ScrollWindowClass::SetScroll(uint32_t aWinWidth, uint32_t aWinHeight, uint32_t aDispWidth, uint32_t aDispHeight)
{
   DispWidth = aDispWidth;
   DispHeight = aDispHeight;
   WinWidth = aWinWidth;
   WinHeight = aWinHeight;

   SCROLLINFO ScrollInfo;
   memset(&ScrollInfo, 0, sizeof(ScrollInfo));
   ScrollInfo.cbSize = sizeof(ScrollInfo);
   ScrollInfo.fMask = SIF_RANGE | SIF_POS;
   if(DispWidth > WinWidth)
      ScrollInfo.nMax = DispWidth - WinWidth;
   else
      ScrollInfo.nMax = 0;
   ScrollInfo.nPage = ScrollInfo.nMax/4;
   SetScrollInfo(WindowHandle, SB_HORZ, &ScrollInfo, TRUE);
   if(DispHeight > WinHeight)
      ScrollInfo.nMax = DispHeight - WinHeight;
   else
      ScrollInfo.nMax = 0;
   ScrollInfo.nPage = ScrollInfo.nMax/4;
   SetScrollInfo(WindowHandle, SB_VERT, &ScrollInfo, TRUE);
   XOffset = 0;
   YOffset = 0;
}

void ScrollWindowClass::OnVScroll(int code, int pos)
{
  SCROLLINFO ScrollInfo;
  int Line, Page;

  memset(&ScrollInfo, 0, sizeof(ScrollInfo));
  ScrollInfo.cbSize = sizeof(ScrollInfo);
  ScrollInfo.fMask = SIF_POS | SIF_RANGE;
  GetScrollInfo(WindowHandle, SB_VERT, &ScrollInfo);
  Line = WinHeight/10;
  if(!Line)
    Line = 1;
  Page = WinHeight*9/10;
  if(!Page)
    Page = 1;
  switch(code)
  {
    case SB_LINEUP:
      ScrollInfo.nPos -= Line;
      break;
    case SB_LINEDOWN:
      ScrollInfo.nPos += Line;
      break;
    case SB_PAGEDOWN:
      ScrollInfo.nPos += Page;
      break;
    case SB_PAGEUP:
      ScrollInfo.nPos -= Page;
      break;
    case SB_THUMBPOSITION:
      ScrollInfo.nPos = pos;
      break;
    case SB_ENDSCROLL:
      InvalidateRect(WindowHandle, 0, TRUE);
    default:
      return;
  }
  if(ScrollInfo.nPos < 0)
     ScrollInfo.nPos = 0;
  else if(ScrollInfo.nPos > ScrollInfo.nMax)
     ScrollInfo.nPos = ScrollInfo.nMax;

  SetScrollInfo(WindowHandle, SB_VERT, &ScrollInfo, TRUE);
  YOffset = ScrollInfo.nPos;
}

void ScrollWindowClass::OnHScroll(int code, int pos)
{
  SCROLLINFO ScrollInfo;
  int Line, Page;
  memset(&ScrollInfo, 0, sizeof(ScrollInfo));
  ScrollInfo.cbSize = sizeof(ScrollInfo);
  ScrollInfo.fMask = SIF_POS | SIF_RANGE;
  GetScrollInfo(WindowHandle, SB_HORZ, &ScrollInfo);
  Line = WinWidth/10;
  if(!Line)
    Line = 1;
  Page = WinWidth*9/10;
  if(!Page)
    Page = 1;
  switch(code)
  {
    case SB_LINELEFT:
      ScrollInfo.nPos -= Line;
      break;
    case SB_LINERIGHT:
      ScrollInfo.nPos += Line;
      break;
    case SB_PAGERIGHT:
      ScrollInfo.nPos += Page;
      break;
    case SB_PAGELEFT:
      ScrollInfo.nPos -= Page;
      break;
    case SB_THUMBPOSITION:
      ScrollInfo.nPos = pos;
      break;
    case SB_ENDSCROLL:
      InvalidateRect(WindowHandle, 0, TRUE);
    default:
      return;
  }
  if(ScrollInfo.nPos < 0)
    ScrollInfo.nPos = 0;
  else if(ScrollInfo.nPos > ScrollInfo.nMax)
    ScrollInfo.nPos = ScrollInfo.nMax;

  XOffset = ScrollInfo.nPos;
  SetScrollInfo(WindowHandle, SB_HORZ, &ScrollInfo, TRUE);
}
