#ifndef RESIZEDIALOG_H_INC
#define RESIZEDIALOG_H_INC

#include "dialog.h"

class SizeDialogClass : public ModalDialogClass
{
private:
   virtual bool OnCommand(UINT Id, UINT Msg, HWND CtrlHwnd)
   {
      switch(Id)
      {
      case IDC_SIZE_X:
         if(Msg == EN_CHANGE && Lock)
         {
            Lock = false; // Prevent a EN_CHANGE for IDC_SIZE_Y caused by setting it's text
            int32_t t;
            GetDlgItemInt(IDC_SIZE_X, t);
            SetDlgItemInt(IDC_SIZE_Y, int32_t(t/Ratio));
            Lock = true;
         }
         break;
      case IDC_SIZE_Y:
         if(Msg == EN_CHANGE && Lock)
         {
            Lock = false; // Prevent a EN_CHANGE for IDC_SIZE_X caused by setting it's text
            int32_t t;
            GetDlgItemInt(IDC_SIZE_Y, t);
            SetDlgItemInt(IDC_SIZE_X, int32_t(t*Ratio));
            Lock = true;
         }
         break;
      case IDC_SIZE_LOCK:
         if(Msg == BN_CLICKED)
         {
            if(SendMessage(CtrlHwnd, BM_GETCHECK, 0, 0) != BST_UNCHECKED)
            {
               int32_t t;
               GetDlgItemInt(IDC_SIZE_X, t);
               SetDlgItemInt(IDC_SIZE_Y, int32_t(t/Ratio));
               Lock = true;
            }
            else
            {
               Lock = false;
            }
         }
         break;
      }
      return TRUE;
   }

   virtual void OnInitDialog()
   {
      SendDlgItemMessage(IDC_SIZE_SPINX, UDM_SETRANGE, 0, MakeLong(30000, 1));
      SendDlgItemMessage(IDC_SIZE_SPINY, UDM_SETRANGE, 0, MakeLong(30000, 1));
      SetDlgItemInt(IDC_SIZE_X, x);
      SetDlgItemInt(IDC_SIZE_Y, y);
      SendDlgItemMessage(IDC_SIZE_LOCK, BM_SETCHECK, BST_CHECKED, 0);
      if(!CanUnlock)
      {
         HWND LockWnd = GetDlgItem(IDC_SIZE_LOCK);
         SetWindowLong(LockWnd, GWL_STYLE, (GetWindowLong(LockWnd, GWL_STYLE) & ~WS_VISIBLE) | WS_DISABLED);
      }
      Lock = true;
   }
   virtual bool OnClose(UINT msg)
   {
      if(msg == IDOK)
      {
         GetDlgItemInt(IDC_SIZE_X, x);
         GetDlgItemInt(IDC_SIZE_Y, y);
      }
      return TRUE;
   }

public:
   SizeDialogClass(HWND ParentWnd, bool canUnlock = true)
   {
      Parent = ParentWnd;
      CanUnlock = canUnlock;
   }
   virtual ~SizeDialogClass()
   {
   }

   UINT Run(uint32_t CurrentX, uint32_t CurrentY)
   {
      x = CurrentX;
      y = CurrentY;
      Ratio = 1.0*x/y;
      return ModalDialogClass::DoDialog(this, Parent, IDD_SIZE_DIALOG);
   }
   int32_t GetX() { return x; }
   int32_t GetY() { return y; }
private:
   int32_t x, y;

   HWND Parent;
   double Ratio;
   bool Lock;
   bool CanUnlock;
};

#endif
